import random
from django.http import HttpResponse, JsonResponse
from django.shortcuts import redirect, render,  get_object_or_404
from mainApp.forms import CategoryForm, CurrencyForm, SlideForm
from mainApp.models import *
from django.shortcuts import render, redirect
from django.http import JsonResponse
from django.contrib import messages
from django.contrib.sessions.backends.db import SessionStore
from datetime import datetime, timedelta, timezone
from django.contrib.auth.decorators import login_required
from django.contrib.auth import authenticate, login as auth_login
from django.contrib.auth import logout
from django.core.mail import send_mail
from django.conf import settings
from twilio.rest import Client
from django.core.mail import EmailMultiAlternatives
from django.template.loader import render_to_string
from django.utils.html import strip_tags
# Create your views here.


def superuser_required(view_func):
    @login_required
    def _wrapped_view(request, *args, **kwargs):
        if not request.user.is_superuser:
            return render(request, 'forbidden.html')
        return view_func(request, *args, **kwargs)
    return _wrapped_view

def login(request):
    error_message = None
    if request.method == 'POST':
        username = request.POST.get('username')
        password = request.POST.get('password')
        user = authenticate(username=username, password=password)
        if user is not None:
            if user.is_staff:
                auth_login(request, user)
                return redirect('admin-luxebeauty')
            else:
                return redirect('index')
        else:
            error_message = 'Invalid username or password'
    return render(request, 'login.html', {'error_message': error_message})

def logout_view(request):
    logout(request)
    return redirect('login')
def index(request):
    categories = Categories.objects.all()
    newproducts = Products.objects.order_by('-added_at')[:10]
    sellers = Products.objects.filter(bestseller=True)[:10] 
    hotkit = Products.objects.filter(hotkit=True)[:10] 
    bestliquid = Products.objects.filter(bestliquid=True)[:10] 
    hotsellingtool = Products.objects.filter(hotsellingtool=True)[:10] 
    banners = SlideImage.objects.all()

    context = {
        'categories' : categories,
        'newproducts' : newproducts,
        'sellers' : sellers,
        'banners' : banners,
        'hotkit' : hotkit,
        'bestliquid' : bestliquid,
        'hotsellingtool' : hotsellingtool,
    }
    return render(request, 'index.html',context)

def bestsellers(request):
    sellers = Products.objects.filter(bestseller=True) 
    context = {
        'sellers': sellers,
    }
    return render(request, 'top20.html',context)

def details(request, product_id):
    product = get_object_or_404(Products, pk=product_id)
    product_images = ProductImage.objects.filter(product=product)
    similar_products = Products.objects.filter(category=product.category).exclude(id=product.id)[:4]
    
    context = {
        'product': product,
        'product_images': product_images,
        'similar_products': similar_products,
    }
    
    return render(request, 'description.html', context)
#  cart views
def add_to_cart(request, product_id):
    product = get_object_or_404(Products, pk=product_id)
    session_key = request.session.session_key
    cart, created = CartItem.objects.get_or_create(session_key=session_key, product=product)

    if not created:
        cart.quantity += 1
        cart.save()

    # Return a JSON response with the message and status
    response_data = {'message': 'Product added to the cart successfully', 'status': 'success'}
    return JsonResponse(response_data)

def get_cart_count(request):
    session_key = request.session.session_key
    cart_count = CartItem.objects.filter(session_key=session_key).count()
    return JsonResponse({'count': cart_count})

def cart(request):
    session_key = request.session.session_key
    cart_items = CartItem.objects.filter(session_key=session_key)
    cart_items_count = sum(item.quantity for item in cart_items)
    # Calculate total price for each item in the cart
    for item in cart_items:
        item.total_price = item.product.price * item.quantity
    # Calculate the final total of all products
    total_price = sum(item.total_price for item in cart_items)
    request.session['cart_items_count'] = cart_items_count
    categories = Categories.objects.all()
    context = {
        'cart_items': cart_items,
        'total_price': total_price,
        'categories': categories,
    }
    return render(request, 'cart.html',context)
def checkout(request):
    session_key = request.session.session_key
    cart_items = CartItem.objects.filter(session_key=session_key)
    categories = Categories.objects.all()
    # Calculate total price for each item in the cart
    for item in cart_items:
        item.total_price = item.product.price * item.quantity

    # Calculate the final total of all products
    total_price = sum(item.total_price for item in cart_items)

    context = {
        'cart_items': cart_items,
        'total_price': total_price,
        'categories': categories,
    }

    return render(request, 'checkout.html', context)

def delete_item(request):
    if request.method == 'POST':
        item_id = request.POST.get('item_id')
        CartItem.objects.filter(id=item_id).delete()
    return redirect('cart')
def send_sms_notification(order_type):
    client = Client(settings.TWILIO_ACCOUNT_SID, settings.TWILIO_AUTH_TOKEN)

    message_body = f'Hello, a new {order_type} order has been created. Check the admin panel for details.'
    message = client.messages.create(
        body=message_body,
        from_=settings.TWILIO_PHONE_NUMBER,
        to='+2567537925776' 
    )
def place_order(request):
    session_key = request.session.session_key
    cart_items = CartItem.objects.filter(session_key=session_key)
    for item in cart_items:
        item.total_price = item.product.price * item.quantity
    total_price = sum(item.total_price for item in cart_items)
    # Create the order
    order = Order.objects.create(
        first_name=request.POST.get('first_name'),
        last_name=request.POST.get('last_name'),
        email=request.POST.get('email'),
        phone=request.POST.get('phone'),
        address=request.POST.get('address'),
        city=request.POST.get('city'),
        country=request.POST.get('country'),
        total_price=total_price
    )
    order.save() 
    for cart_item in cart_items:
        OrderItem.objects.create(
            order=order,
            product=cart_item.product,
            quantity=cart_item.quantity,
            image=cart_item.product.image,
            price=cart_item.product.price
        )

    cart_items.delete()

    html_content = render_to_string('order_email_template.html', {'order': order})
    text_content = strip_tags(html_content)  # Remove HTML tags for plain text email

    # Send email
    subject = 'New Order Created'
    from_email = settings.DEFAULT_FROM_EMAIL
    recipient_list = ['bionluxe@gmail.com']

    email = EmailMultiAlternatives(subject, text_content, from_email, recipient_list)
    email.attach_alternative(html_content, "text/html")
    email.send()
    # send_sms_notification('Order')
    messages.success(request, 'Your Order has been received, We shall contact you soon')
    return redirect('index')

def buy_now(request):
    order2 = BuyNow2.objects.create(
        name = request.POST.get('name'),
        phone_number = request.POST.get('phone_number'),
        product_name = request.POST.get('product_name'),
        product_price = request.POST.get('product_price'),
        product_id = request.POST.get('product_id'),
        product_category = request.POST.get('product_category'),
    )
    order2.save()
    html_content = render_to_string('buy_now_email_template.html', {'order2': order2})
    text_content = strip_tags(html_content) 

    # Send email
    subject = 'New Order Created (Buy Now)'
    from_email = settings.DEFAULT_FROM_EMAIL
    recipient_list = ['bionluxe@gmail.com']

    email = EmailMultiAlternatives(subject, text_content, from_email, recipient_list)
    email.attach_alternative(html_content, "text/html")
    email.send()
    messages.success(request, 'Your Order has been received, We shall contact you soon')
    return redirect('index')

def category_products(request, category_id):
    categories = Categories.objects.all()
    category2 = get_object_or_404(Categories, id=category_id)
    products = Products.objects.filter(category=category2)
    context = {
        'category2': category2,
        'categories': categories,
        'products': products,
        }
    return render(request, 'category_products.html', context)

def search(request):
    return render(request, 'search.html')

def product_search(request):
    query = request.GET.get('q', '')
    products = Products.objects.filter(
        models.Q(product_name__icontains=query)
        # models.Q(category__name__icontains=query) | 
        # models.Q(brand__name__icontains=query) 
    )
    categories = Categories.objects.all()

    context = {
        'products': products,
        'query': query,
        'categories': categories,
    }

    return render(request, 'search-results.html', context)



def add_to_wishlist(request, product_id):
    product = get_object_or_404(Products, pk=product_id)
    session_key = request.session.session_key
    wishlist_item, created = WishlistItem.objects.get_or_create(session_key=session_key, product=product)

    if not created:
        response_data = {'message': 'Product is already in your wishlist', 'status': 'info'}
    else:
        response_data = {'message': 'Product added to the wishlist successfully', 'status': 'success'}

    return JsonResponse(response_data)

def get_wishlist_count(request):
    session_key = request.session.session_key
    wishlist_count = WishlistItem.objects.filter(session_key=session_key).count()
    return JsonResponse({'count': wishlist_count})

def wishlist(request):
    session_key = request.session.session_key
    wishlist_items = WishlistItem.objects.filter(session_key=session_key)
    categories = Categories.objects.all()
    context = {
        'wishlist_items': wishlist_items,
        'categories': categories,
    }
    return render(request, 'wishlist.html', context)


def category_products(request, category_id):
    category = get_object_or_404(Categories, pk=category_id)
    products = Products.objects.filter(category=category)
    context = {
        'category': category,
        'products': products,
    }
    return render(request, 'category_products.html', context)




    # admin side views
@login_required
def admin1(request):
    products2 = Products.objects.all()
    categories = Categories.objects.all()
    slides = SlideImage.objects.all()
    orders = Order.objects.all()
    buynow = BuyNow2.objects.all()
    context = {
        'categories': categories,
        'orders': orders,
        'products': products2,
        'buynow': buynow,
        'slides': slides,
         }   
    return render (request, 'admin1/index.html', context)

@login_required
def slidesAdmin1(request):
    slides = SlideImage.objects.all()
    context = {
        'slides': slides,
         }   
    return render (request, 'admin1/slides.html', context)

@login_required
def create_product(request):
    categories = Categories.objects.all()
    
    if request.method == 'POST':
        name = request.POST.get('name')
        category_id = request.POST.get('category')
        price = request.POST.get('price')
        description = request.POST.get('description')
        discount_amount = request.POST.get('discount_amount')
        discount_percent = request.POST.get('discount_percent')
        bestseller = request.POST.get('bestseller') == 'on'  
        hotkit = request.POST.get('hotkit') == 'on'  
        bestliquid = request.POST.get('bestliquid') == 'on' 
        hotsellingtool = request.POST.get('hotsellingtool') == 'on'  

        # Convert empty strings to None for nullable fields
        price = int(price) if price else None
        discount_amount = int(discount_amount) if discount_amount else None
        discount_percent = float(discount_percent) if discount_percent else None

        main_image = request.FILES.get('main_image')

        # Create the product instance
        product = Products.objects.create(
            product_name=name,
            category_id=category_id,
            price=price,
            description=description,
            discount_amount=discount_amount,
            discount_percent=discount_percent,
            image=main_image,
            bestseller=bestseller,  
            hotkit=hotkit,  
            bestliquid=bestliquid,  
            hotsellingtool=hotsellingtool,  
        )

        # Handling additional images
        additional_images = request.FILES.getlist('images[]')
        for image in additional_images:
            ProductImage.objects.create(product=product, image=image)

        messages.success(request, 'Product added successfully!')
        return redirect('create_product')  
    
    context = {
        'categories': categories
    }
    return render(request, 'admin1/products.html', context)

@login_required
def productList(request):
    products = Products.objects.all()
    
    context = {
        'products': products,
         }   
    return render (request, 'admin1/viewproduct.html', context)
@login_required
def categoryList(request):
    categories = Categories.objects.all()
    context = {
        'categories': categories,
         }   
    return render (request, 'admin1/categoryList.html', context)
@login_required
def delete_product(request):
    if request.method == 'POST':
        product_id = request.POST.get('product_id')
        product = Products.objects.get(id=product_id)
        product.delete()
    messages.success(request, 'Product Deleted Successfully')
    return redirect('productList')
@login_required
def delete_category(request):
    if request.method == 'POST':
        category_id = request.POST.get('category_id')
        category = Categories.objects.get(id=category_id)
        category.delete()
    messages.success(request, 'Category Deleted Successfully')
    return redirect('categoryList')
@login_required
def add_category(request):
    if request.method == 'POST':
        form = CategoryForm(request.POST, request.FILES)
        if form.is_valid():
            form.save()
            return redirect('categoryList')  
    else:
        form = CategoryForm()
    return render(request, 'admin1/add_category.html', {'form': form})
@login_required
def order_list(request):
    orders = Order.objects.order_by('-created_at')
    items = BuyNow2.objects.all()
    current_date = datetime.now()
    one_day_ago = current_date - timedelta(days=1)

    return render(request, 'admin1/orders.html', {
        'orders': orders, 
        'items': items,
        'one_day_ago': one_day_ago
        }
        )

def delete_order_view(request, pk):
    order = get_object_or_404(BuyNow2, pk=pk)
    if request.method == 'POST':
        order.delete()
        messages.success(request, 'Order Deleted Successfully')
        return redirect('order_list')  



def delete_order(request):
    if request.method == 'POST':
        order_id = request.POST.get('order_id')
        order = Order.objects.get(id=order_id)
        order.delete()
    messages.success(request, 'Order Deleted Successfully')
    return redirect('order_list')

def order_detail(request, order_id):
    order = get_object_or_404(Order, id=order_id)
    return render(request, 'admin1/orderDetails.html', {'order': order})


def edit_product(request, product_id):
    product = get_object_or_404(Products, id=product_id)
    categories = Categories.objects.all()
    product_images = product.productimage_set.all()

    if request.method == 'POST':
        product_name = request.POST.get('product_name')
        category_id = request.POST.get('category')
        price = request.POST.get('price')
        description = request.POST.get('description')
        discount_amount = request.POST.get('discount_amount')
        discount_percent = request.POST.get('discount_percent')
        bestseller = request.POST.get('bestseller') == 'on'
        hotkit = request.POST.get('hotkit') == 'on'
        bestliquid = request.POST.get('bestliquid') == 'on'
        hotsellingtool = request.POST.get('hotsellingtool') == 'on'

        # Convert empty strings to None for nullable fields
        price = int(price) if price else None
        discount_amount = int(discount_amount) if discount_amount else None
        discount_percent = float(discount_percent) if discount_percent else None

        main_image = request.FILES.get('main_image')

        # Fetch the category instance
        category = get_object_or_404(Categories, id=category_id)

        # Update existing product data
        product.product_name = product_name
        product.category = category
        product.price = price
        product.description = description
        product.discount_amount = discount_amount
        product.discount_percent = discount_percent
        product.bestseller = bestseller
        product.hotkit = hotkit
        product.bestliquid = bestliquid
        product.hotsellingtool = hotsellingtool

        # Handle main image update
        if main_image:
            product.image = main_image

        # Handle additional images
        additional_images = request.FILES.getlist('images[]')
        for image in additional_images:
            ProductImage.objects.create(product=product, image=image)

        # Save the updated product
        product.save()

        messages.success(request, 'Product edited successfully!')
        return redirect('productList')

    context = {
        'product': product,
        'categories': categories,
        'product_images': product_images
    }
    return render(request, 'admin1/edit_product.html', context)
def error(request):
    return render(request, '404.html')

def add_currency(request):
    if request.method == 'POST':
        form = CurrencyForm(request.POST)
        if form.is_valid():
            form.save()
            messages.success(request, 'Currency added Successfully')
            return redirect('productList')
    else:
        form = CurrencyForm()
    return render(request, 'admin1/add_currency.html', {'form': form})


@login_required
def add_slide(request):
    if request.method == 'POST':
        form = SlideForm(request.POST, request.FILES)
        if form.is_valid():
            form.save()
            return redirect('slides')  
    else:
        form = SlideForm()
    return render(request, 'admin1/add-slides.html', {'form': form})
@login_required
def delete_slide(request):
    if request.method == 'POST':
        slide_id = request.POST.get('slide_id')
        slide = SlideImage.objects.get(id=slide_id)
        slide.delete()
    messages.success(request, 'Slide Deleted Successfully')
    return redirect('slides')
